@@ -118,9 +118,14 @@ dialog_show_callback (GtkWidget *widget, gpointer callback_data)
 
 	password_dialog = EEL_PASSWORD_DIALOG (callback_data);
 
-	/* Move the focus to the password entry */
-	eel_caption_table_entry_grab_focus (EEL_CAPTION_TABLE (password_dialog->details->table), 
-					    CAPTION_TABLE_PASSWORD_ROW);
+	if (password_dialog->details->readonly_username == FALSE) {
+		eel_caption_table_entry_grab_focus (EEL_CAPTION_TABLE (password_dialog->details->table), 
+						    CAPTION_TABLE_USERNAME_ROW);
+	}
+	else {
+		eel_caption_table_entry_grab_focus (EEL_CAPTION_TABLE (password_dialog->details->table), 
+						    CAPTION_TABLE_PASSWORD_ROW);
+	}
 }
 
 static void
@@ -295,6 +300,8 @@ eel_password_dialog_set_readonly_username (EelPasswordDialog	*password_dialog,
 {
 	g_return_if_fail (password_dialog != NULL);
 	g_return_if_fail (EEL_IS_PASSWORD_DIALOG (password_dialog));
+
+	password_dialog->details->readonly_username = readonly;
 	
 	eel_caption_table_set_entry_readonly (EEL_CAPTION_TABLE (password_dialog->details->table),
 						   CAPTION_TABLE_USERNAME_ROW,
