@@ -4,6 +4,7 @@
                             a single eel preference key.
 
    Copyright (C) 1999, 2000 Eazel, Inc.
+   Copyright (C) 2002 Bent Spoon Software
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -26,13 +27,13 @@
 #include <config.h>
 #include "eel-preferences-item.h"
 
-#include "eel-preferences.h"
 #include "eel-art-gtk-extensions.h"
 #include "eel-enumeration.h"
 #include "eel-font-picker.h"
 #include "eel-glib-extensions.h"
 #include "eel-gtk-extensions.h"
-#include "eel-gtk-macros.h"
+#include "eel-marshal.h"
+#include "eel-preferences.h"
 #include "eel-radio-button-group.h"
 #include "eel-string-picker.h"
 #include "eel-string.h"
@@ -41,6 +42,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <libgnome/gnome-macros.h>
 
 #define PREFERENCES_ITEM_UNDEFINED_ITEM -1U
 
@@ -68,94 +70,37 @@ struct EelPreferencesItemDetails
 typedef enum
 {
 	CUSTOM_UPDATE_DISPLAYED_VALUE,
+	CUSTOM_DESCRIPTION_CHANGED,
 	LAST_SIGNAL
 } EelStringListSignals;
 
-static guint preferences_item_signals[LAST_SIGNAL];
+static guint signals[LAST_SIGNAL];
 
-/* GtkObjectClass methods */
-static void eel_preferences_item_initialize_class                     (EelPreferencesItemClass *preferences_item_class);
-static void eel_preferences_item_initialize                           (EelPreferencesItem      *preferences_item);
-static void preferences_item_destroy                                  (GtkObject               *object);
+static void preferences_item_update_displayed_value                   (EelPreferencesItem  *item);
+static void preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem  *item);
+static void preferences_item_update_text_settings_at_idle             (EelPreferencesItem  *item);
+static void preferences_item_update_description                       (EelPreferencesItem  *item);
+static void enumeration_radio_changed_callback                        (EelRadioButtonGroup *radio_button_group,
+								       gpointer             callback_data);
+static void boolean_button_toggled_callback                           (GtkWidget           *button_group,
+								       gpointer             user_data);
+static void editable_string_changed_callback                          (GtkWidget           *caption,
+								       gpointer             user_data);
+static void editable_integer_changed_callback                         (GtkWidget           *caption,
+								       gpointer             user_data);
+static void enumeration_menu_changed_callback                         (EelStringPicker     *string_picker,
+								       EelPreferencesItem  *item);
+static void enumeration_list_changed_callback                         (EelStringPicker     *string_picker,
+								       EelPreferencesItem  *item);
 
-/* Private stuff */
-static void preferences_item_create_boolean                           (EelPreferencesItem      *item);
-static void preferences_item_create_editable_integer                  (EelPreferencesItem      *item);
-static void preferences_item_create_editable_string                   (EelPreferencesItem      *item);
-static void preferences_item_create_enumeration_menu                  (EelPreferencesItem      *item);
-static void preferences_item_create_enumeration_radio                 (EelPreferencesItem      *item,
-								       gboolean                 horizontal);
-static void preferences_item_create_enumeration_list                  (EelPreferencesItem      *item,
-								       gboolean                 horizontal);
-static void preferences_item_create_font                              (EelPreferencesItem      *item);
-static void preferences_item_create_font                              (EelPreferencesItem      *item);
-static void preferences_item_create_padding                           (EelPreferencesItem      *item);
-static void preferences_item_create_smooth_font                       (EelPreferencesItem      *item);
-static void preferences_item_update_displayed_value                   (EelPreferencesItem      *preferences_item);
-static void preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem      *preferences_item);
-static void preferences_item_update_text_settings_at_idle             (EelPreferencesItem      *preferences_item);
-
-/* User triggered item changed callbacks */
-static void enumeration_radio_changed_callback                        (EelRadioButtonGroup     *radio_button_group,
-								       gpointer                 callback_data);
-static void boolean_button_toggled_callback                           (GtkWidget               *button_group,
-								       gpointer                 user_data);
-static void editable_string_changed_callback                          (GtkWidget               *caption,
-								       gpointer                 user_data);
-static void editable_integer_changed_callback                         (GtkWidget               *caption,
-								       gpointer                 user_data);
-static void enumeration_menu_changed_callback                         (EelStringPicker         *string_picker,
-								       EelPreferencesItem      *item);
-static void font_changed_callback                                     (GtkWidget               *caption,
-								       gpointer                 user_data);
-static void enumeration_list_changed_callback                         (EelStringPicker         *string_picker,
-								       EelPreferencesItem      *item);
-static void smooth_font_changed_callback                              (EelFontPicker           *font_picker,
-								       gpointer                 callback_data);
-static void custom_changed_callback                                   (GtkWidget               *custom,
-								       gpointer                 user_data);
-
-EEL_DEFINE_CLASS_BOILERPLATE (EelPreferencesItem, eel_preferences_item, GTK_TYPE_VBOX)
-
-/* EelPreferencesItemClass methods */
-static void
-eel_preferences_item_initialize_class (EelPreferencesItemClass *preferences_item_class)
-{
-	GtkObjectClass *object_class;
-
-	object_class = GTK_OBJECT_CLASS (preferences_item_class);
-
-	/* GtkObjectClass */
-	object_class->destroy = preferences_item_destroy;
-
-	/* Signals */
-	preferences_item_signals[CUSTOM_UPDATE_DISPLAYED_VALUE] = 
-		gtk_signal_new ("custom_update_displayed_value",
-				GTK_RUN_LAST,
-				object_class->type,
-				0,
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 
-				0);
-
-	gtk_object_class_add_signals (object_class, preferences_item_signals, LAST_SIGNAL);
-}
-
-static void
-eel_preferences_item_initialize (EelPreferencesItem *item)
-{
-	item->details = g_new0 (EelPreferencesItemDetails, 1);
-	item->details->item_type = PREFERENCES_ITEM_UNDEFINED_ITEM;
-	item->details->enumeration_list_unique_exceptions = eel_string_list_new (TRUE);
-}
+GNOME_CLASS_BOILERPLATE (EelPreferencesItem, eel_preferences_item,
+			 GtkVBox, GTK_TYPE_VBOX)
 
 /* GtkObjectClass methods */
 static void
-preferences_item_destroy (GtkObject *object)
+preferences_item_finalize (GObject *object)
 {
-	EelPreferencesItem * item;
-	
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (object));
+	EelPreferencesItem *item;
 	
 	item = EEL_PREFERENCES_ITEM (object);
 
@@ -165,8 +110,7 @@ preferences_item_destroy (GtkObject *object)
 	eel_string_list_free (item->details->enumeration_list_unique_exceptions);
 	g_free (item->details);
 
-	/* Chain destroy */
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 /*
@@ -323,9 +267,7 @@ preferences_item_update_custom (EelPreferencesItem *item)
 {
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 
-	gtk_signal_emit (GTK_OBJECT (item),
-			 preferences_item_signals[CUSTOM_UPDATE_DISPLAYED_VALUE]);
-	
+	g_signal_emit (G_OBJECT (item), signals[CUSTOM_UPDATE_DISPLAYED_VALUE], 0);
 }
 
 /* This callback is called whenever the preference value changes, so that we can
@@ -352,7 +294,7 @@ preferences_item_set_main_child (EelPreferencesItem *item,
 		eel_preferences_add_callback_while_alive (item->details->preference_name,
 							  preferences_item_value_changed_callback,
 							  item,
-							  GTK_OBJECT (item));
+							  G_OBJECT (item));
 	}
 
 	gtk_box_pack_start (GTK_BOX (item),
@@ -364,6 +306,8 @@ preferences_item_set_main_child (EelPreferencesItem *item,
 	gtk_widget_show (child);
 
 	item->details->main_child = child;
+
+	preferences_item_update_description (item);
 }
 
 static void
@@ -381,7 +325,7 @@ preferences_item_add_connection_child (EelPreferencesItem *item,
 
 	connection = g_new0 (PreferencesItemConnection, 1);
 	connection->widget = child;
-	connection->signal_id = gtk_signal_connect (GTK_OBJECT (child),
+	connection->signal_id = g_signal_connect (child,
 						    signal_name,
 						    signal,
 						    item);
@@ -422,7 +366,7 @@ preferences_item_create_enumeration_radio (EelPreferencesItem *item,
 	preferences_item_add_connection_child (item,
 					       child,
 					       "changed",
-					       GTK_SIGNAL_FUNC (enumeration_radio_changed_callback));
+					       G_CALLBACK (enumeration_radio_changed_callback));
 
 	preferences_item_set_main_child (item, child);
 }
@@ -434,9 +378,9 @@ preferences_item_create_enumeration_list (EelPreferencesItem *item,
  	guint i;
  	guint j;
 	char *enumeration_id;
-	char *description;
 	char *enum_description;
-	EelStringList *default_value;
+	GConfValue *fallback;
+	EelStringList *default_value, *string_list;
 	guint num_pickers;
 	GtkWidget *string_picker;
 	GtkWidget *picker_box;
@@ -446,23 +390,25 @@ preferences_item_create_enumeration_list (EelPreferencesItem *item,
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (eel_strlen (description) > 0);
-
 	enumeration_id = eel_preferences_get_enumeration_id (item->details->preference_name);
 	g_return_if_fail (eel_strlen (enumeration_id) > 0);
 	g_return_if_fail (eel_enumeration_id_get_length (enumeration_id) > 0);
 
-	/* FIXME: Hard coded user level */
-	default_value = eel_preferences_default_get_string_list (item->details->preference_name, 0);
+	fallback = eel_preferences_get_emergency_fallback (item->details->preference_name);
 
+	g_assert (fallback);
+
+	default_value = eel_gconf_value_get_eel_string_list (fallback);
+
+	gconf_value_free (fallback);
+	
 	num_pickers = eel_string_list_get_length (default_value);
 	g_return_if_fail (num_pickers > 0);
 	
 	child = gtk_vbox_new (FALSE, 4);
 	picker_box = horizontal ? gtk_hbox_new (FALSE, 4): gtk_vbox_new (FALSE, 4);
 
-	title = gtk_label_new (description);
+	title = gtk_label_new ("");
 	gtk_misc_set_alignment (GTK_MISC (title), 0.0, 0.5);
 	gtk_label_set_justify (GTK_LABEL (title), GTK_JUSTIFY_LEFT);
 
@@ -485,19 +431,23 @@ preferences_item_create_enumeration_list (EelPreferencesItem *item,
 	for (j = 0; j < num_pickers; j++) {
 		string_picker = eel_string_picker_new ();
 		eel_caption_set_show_title (EEL_CAPTION (string_picker), FALSE);
-		
+
+		string_list = eel_string_list_new (TRUE);
 		for (i = 0; i < eel_enumeration_id_get_length (enumeration_id); i++) {
 			enum_description = eel_enumeration_id_get_nth_description_translated (enumeration_id, i);
 			g_assert (enum_description != NULL);
 
 			if (enum_description[0] == '-') {
-				eel_string_picker_insert_separator (EEL_STRING_PICKER (string_picker));
+				eel_string_list_insert (string_list, EEL_STRING_PICKER_SEPARATOR_STRING);
 			} else {
-				eel_string_picker_insert_string (EEL_STRING_PICKER (string_picker), enum_description);
+				eel_string_list_insert (string_list, enum_description);
 			}
 			g_free (enum_description);
 		}
 
+		eel_string_picker_set_string_list (EEL_STRING_PICKER (string_picker), string_list);
+		eel_string_list_free (string_list);
+		
 		gtk_box_pack_start (GTK_BOX (picker_box),
 				    string_picker,
 				    FALSE,
@@ -509,7 +459,7 @@ preferences_item_create_enumeration_list (EelPreferencesItem *item,
 		preferences_item_add_connection_child (item,
 						       string_picker,
 						       "changed",
-						       GTK_SIGNAL_FUNC (enumeration_list_changed_callback));
+						       G_CALLBACK (enumeration_list_changed_callback));
 	}
  	g_free (enumeration_id);
 
@@ -531,23 +481,18 @@ preferences_item_update_boolean (EelPreferencesItem *item)
 static void
 preferences_item_create_boolean (EelPreferencesItem *item)
 {
-	char *description;
 	GtkWidget *child;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (eel_strlen (description) > 0);
-
-	child = gtk_check_button_new_with_label (description);
+	child = gtk_check_button_new_with_label ("");
 	gtk_label_set_justify (GTK_LABEL (GTK_BIN (child)->child), GTK_JUSTIFY_LEFT);
-	g_free (description);
 	
 	preferences_item_add_connection_child (item,
 					       child,
 					       "toggled",
-					       GTK_SIGNAL_FUNC (boolean_button_toggled_callback));
+					       G_CALLBACK (boolean_button_toggled_callback));
 				      
 	preferences_item_set_main_child (item, child);
 }
@@ -575,33 +520,27 @@ preferences_item_update_editable_string (EelPreferencesItem *item)
 static void
 preferences_item_create_editable_string (EelPreferencesItem *item)
 {
-	char *description;
 	GtkWidget *child;
 	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_assert (description != NULL);
-	
 	child = eel_text_caption_new ();
 
  	/* FIXME:
 	 * This is a Nautilus specific special case for the home uri preference,
 	 * in the future this should be generalized.
 	 */
- 	if (g_strcasecmp (item->details->preference_name, "preferences/home_uri") == 0) {
+ 	if (g_ascii_strcasecmp (item->details->preference_name, "preferences/home_uri") == 0) {
  		eel_text_caption_set_expand_tilde (EEL_TEXT_CAPTION (child), TRUE);
  	}
 	
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-
-	g_free (description);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 	
 	preferences_item_add_connection_child (item,
 					       child,
 					       "changed",
-					       GTK_SIGNAL_FUNC (editable_string_changed_callback));
+					       G_CALLBACK (editable_string_changed_callback));
 
 	preferences_item_set_main_child (item, child);
 }
@@ -630,26 +569,19 @@ preferences_item_update_editable_integer (EelPreferencesItem *item)
 static void
 preferences_item_create_editable_integer (EelPreferencesItem *item)
 {
-	char *description;
 	GtkWidget *child;
 	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	
-	g_assert (description != NULL);
-	
 	child = eel_text_caption_new ();
 
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-
-	g_free (description);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 
 	preferences_item_add_connection_child (item,
 					       child,
 					       "changed",
-					       GTK_SIGNAL_FUNC (editable_integer_changed_callback));
+					       G_CALLBACK (editable_integer_changed_callback));
 
 	preferences_item_set_main_child (item, child);
 }
@@ -697,120 +629,45 @@ preferences_item_create_enumeration_menu (EelPreferencesItem *item)
 {
  	guint i;
 	char *enumeration_id;
-	char *description;
 	GtkWidget *child;
+	EelStringList *string_list;
+	char *enum_description;
 	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 	
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (description != NULL);
-	
 	child = eel_string_picker_new ();
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-	g_free (description);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 
 	enumeration_id = eel_preferences_get_enumeration_id (item->details->preference_name);
 	g_return_if_fail (eel_strlen (enumeration_id) > 0);
 	g_return_if_fail (eel_enumeration_id_get_length (enumeration_id) > 0);
+
+	string_list = eel_string_list_new (TRUE);
 	
 	/* Populate the string picker */
 	for (i = 0; i < eel_enumeration_id_get_length (enumeration_id); i++) {
-		description = eel_enumeration_id_get_nth_description_translated (enumeration_id, i);
-		g_assert (description != NULL);
-		
-		if (description[0] == '-') {
-			eel_string_picker_insert_separator (EEL_STRING_PICKER (child));
+		enum_description = eel_enumeration_id_get_nth_description_translated (enumeration_id, i);
+		g_assert (enum_description != NULL);
+
+		if (enum_description[0] == '-') {
+			eel_string_list_insert (string_list, EEL_STRING_PICKER_SEPARATOR_STRING);
 		} else {
-			eel_string_picker_insert_string (EEL_STRING_PICKER (child), description);
+			eel_string_list_insert (string_list, enum_description);
 		}
 		
-		g_free (description);
+		g_free (enum_description);
 	}
  	g_free (enumeration_id);
 
-	preferences_item_add_connection_child (item,
-					       child,
-					       "changed",
-					       GTK_SIGNAL_FUNC (enumeration_menu_changed_callback));
-
-	preferences_item_set_main_child (item, child);
-}
-
-static void
-preferences_item_update_font (EelPreferencesItem *item)
-{
-	char *current_value;
-
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_FONT);
-
-	current_value = eel_preferences_get (item->details->preference_name);
-	g_assert (current_value != NULL);
-
-	/* The value of the gconf preference can be anything.  In theory garbage could
-	 * be used for the preference using a third party tool.  So we make sure that
-	 * it is one of the choice before trying to select it, otherwise we would get
-	 * assertions.
-	 */
-	if (eel_string_picker_contains (EEL_STRING_PICKER (item->details->main_child), current_value)) {
-		eel_string_picker_set_selected_string (EEL_STRING_PICKER (item->details->main_child),
-						       current_value);
-	}
-
-	g_free (current_value);
-}
-
-static void
-preferences_item_create_font (EelPreferencesItem *item)
-{
-	char *description;
-	EelStringList *font_list;
-	GtkWidget *child;
-
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
-
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (description != NULL);
-	
-	child = eel_string_picker_new ();
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-	
-	g_free (description);
-
-	/* FIXME bugzilla.eazel.com 1274: Need to query system for available fonts */
-	font_list = eel_string_list_new (TRUE);
-
-	/* Once upon a time we had a bug in Eel that caused crashes with the "fixed" 
-	 * font.  That bug (2256) was fixed by removing the "fixed" choice from this menu
-	 * below.  Subsequently we fixed many font bugs in eel (such hard coded font sizes)
-	 * that would cause both crashes and ugliness. Bug 2256 seems to have been fixed by
-	 * these changes as well.
-	 *
-	 * Anyhow, the "fixed" font choice is not very interesting because the other fonts 
-	 * look much better.  However, in multi byte locales, the fixed font is usually the
-	 * only one that is available at the right encoding.
-	 */
-
-	/* FIXME bugzilla.eazel.com 7907: 
-	 * The "GTK System Font" string is hard coded in many places.
-	 */
-	eel_string_list_insert (font_list, "GTK System Font");
-	eel_string_list_insert (font_list, "fixed");
-	eel_string_list_insert (font_list, "helvetica");
-	eel_string_list_insert (font_list, "times");
-	eel_string_list_insert (font_list, "courier");
-	eel_string_list_insert (font_list, "lucida");
-
-	eel_string_picker_set_string_list (EEL_STRING_PICKER (child), font_list);
-	eel_string_list_free (font_list);
+	eel_string_picker_set_string_list (EEL_STRING_PICKER (child), string_list);
+	eel_string_list_free (string_list);
 	
 	preferences_item_add_connection_child (item,
 					       child,
 					       "changed",
-					       GTK_SIGNAL_FUNC (font_changed_callback));
-	
+					       G_CALLBACK (enumeration_menu_changed_callback));
+
 	preferences_item_set_main_child (item, child);
 }
 
@@ -823,24 +680,23 @@ preferences_item_create_padding (EelPreferencesItem *item)
 }
 
 static void
-preferences_item_update_smooth_font (EelPreferencesItem *item)
+preferences_item_update_font (EelPreferencesItem *item)
 {
  	char *current_value;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_SMOOTH_FONT);
+	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_FONT);
 
  	current_value = eel_preferences_get (item->details->preference_name);
  	g_assert (current_value != NULL);
-
  	eel_font_picker_set_selected_font (EEL_FONT_PICKER (item->details->main_child),
 					   current_value);
  	g_free (current_value);
 }
 
 static void
-smooth_font_changed_callback (EelFontPicker *font_picker,
-			      gpointer callback_data)
+font_changed_callback (EelFontPicker *font_picker,
+		       gpointer callback_data)
 {
 	EelPreferencesItem *item;
  	char *new_value;
@@ -856,25 +712,18 @@ smooth_font_changed_callback (EelFontPicker *font_picker,
 }
 
 static void
-preferences_item_create_smooth_font (EelPreferencesItem *item)
+preferences_item_create_font (EelPreferencesItem *item)
 {
-	char *description = NULL;
 	GtkWidget *child;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (description != NULL);
-	
-	child = eel_font_picker_new ();
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-	g_free (description);
+	child = eel_font_picker_new (NULL);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 
-	preferences_item_add_connection_child (item,
-					       child,
-					       "changed",
-					       GTK_SIGNAL_FUNC (smooth_font_changed_callback));
+	preferences_item_add_connection_child (item, child, "changed",
+					       G_CALLBACK (font_changed_callback));
 
 	preferences_item_set_main_child (item, child);
 }
@@ -895,7 +744,7 @@ eel_preferences_item_new (const char *preference_name,
 
 	g_return_val_if_fail (eel_strlen (preference_name) > 0, NULL);
 	g_return_val_if_fail (item_type >= EEL_PREFERENCE_ITEM_BOOLEAN, FALSE);
-	g_return_val_if_fail (item_type <= EEL_PREFERENCE_ITEM_SMOOTH_FONT, FALSE);
+	g_return_val_if_fail (item_type <= EEL_PREFERENCE_ITEM_PADDING, FALSE);
 
 	item = EEL_PREFERENCES_ITEM
 		(gtk_widget_new (eel_preferences_item_get_type (), NULL));
@@ -930,10 +779,6 @@ eel_preferences_item_new (const char *preference_name,
 		preferences_item_create_font (item);
 		break;
 	
-	case EEL_PREFERENCE_ITEM_SMOOTH_FONT:
-		preferences_item_create_smooth_font (item);
-		break;
-	
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
 		preferences_item_create_editable_string (item);
 		break;	
@@ -982,7 +827,7 @@ eel_preferences_item_new_custom (const char *preference_name,
 	preferences_item_add_connection_child (item,
 					       child,
 					       signal_name,
-					       GTK_SIGNAL_FUNC (custom_changed_callback));
+					       G_CALLBACK (custom_changed_callback));
 
 	preferences_item_set_main_child (item, child);
 
@@ -1031,27 +876,6 @@ boolean_button_toggled_callback (GtkWidget *button, gpointer user_data)
 	eel_preferences_set_boolean (item->details->preference_name, active_state);
 }
 
-static void
-font_changed_callback (GtkWidget *string_picker, gpointer user_data)
-{
-	EelPreferencesItem *item;
-	char *selected_string;
-
-	g_return_if_fail (EEL_IS_STRING_PICKER (string_picker));
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (user_data));
-
-	item = EEL_PREFERENCES_ITEM (user_data);
-
-	g_return_if_fail (item->details->preference_name != NULL);
-
-	selected_string = eel_string_picker_get_selected_string (EEL_STRING_PICKER (string_picker));
-	g_return_if_fail (selected_string != NULL);
-
-	eel_preferences_set (item->details->preference_name, selected_string);
-
-	g_free (selected_string);
-}
-
 static void
 editable_string_changed_callback (GtkWidget *button, gpointer user_data)
 {
@@ -1171,11 +995,11 @@ enumeration_list_changed_callback (EelStringPicker *string_picker,
 }
 
 char *
-eel_preferences_item_get_name (const EelPreferencesItem *preferences_item)
+eel_preferences_item_get_name (const EelPreferencesItem *item)
 {
-	g_return_val_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item), NULL);
+	g_return_val_if_fail (EEL_IS_PREFERENCES_ITEM (item), NULL);
 
-	return g_strdup (preferences_item->details->preference_name);
+	return g_strdup (item->details->preference_name);
 }
 
 static void
@@ -1197,8 +1021,8 @@ preferences_item_update_displayed_value (EelPreferencesItem *item)
 		connection = node->data;
 		g_assert (GTK_IS_WIDGET (connection->widget));
 		
-		gtk_signal_handler_block (GTK_OBJECT (connection->widget),
-					  connection->signal_id);
+		g_signal_handler_block (G_OBJECT (connection->widget),
+					connection->signal_id);
 	}
 
 	/* Update the child widget according to the item type */
@@ -1222,10 +1046,6 @@ preferences_item_update_displayed_value (EelPreferencesItem *item)
 		preferences_item_update_font (item);
 		break;
 	
-	case EEL_PREFERENCE_ITEM_SMOOTH_FONT:
-		preferences_item_update_smooth_font (item);
-		break;
-	
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
 		preferences_item_update_editable_string (item);
 		break;	
@@ -1253,20 +1073,20 @@ preferences_item_update_displayed_value (EelPreferencesItem *item)
 		connection = node->data;
 		g_assert (GTK_IS_WIDGET (connection->widget));
 		
-		gtk_signal_handler_unblock (GTK_OBJECT (connection->widget),
-					    connection->signal_id);
+		g_signal_handler_unblock (G_OBJECT (connection->widget),
+					  connection->signal_id);
 	}
 }
 
 static gboolean
-update_text_settings_at_idle (EelPreferencesItem *preferences_item)
+update_text_settings_at_idle (EelPreferencesItem *item)
 {
 	char *text;
 
-	text = eel_text_caption_get_text (EEL_TEXT_CAPTION (preferences_item->details->main_child));
+	text = eel_text_caption_get_text (EEL_TEXT_CAPTION (item->details->main_child));
 
 	if (text != NULL) {
-		eel_preferences_set (preferences_item->details->preference_name, text);
+		eel_preferences_set (item->details->preference_name, text);
 		g_free (text);
 	}
 	
@@ -1276,29 +1096,27 @@ update_text_settings_at_idle (EelPreferencesItem *preferences_item)
 }
 
 static void
-preferences_item_update_text_settings_at_idle (EelPreferencesItem *preferences_item)
+preferences_item_update_text_settings_at_idle (EelPreferencesItem *item)
 {
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item));
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 
 	if (text_idle_handler == FALSE) {
-		gtk_idle_add ((GtkFunction) update_text_settings_at_idle, preferences_item);
+		gtk_idle_add ((GtkFunction) update_text_settings_at_idle, item);
 		text_idle_handler = TRUE;
 	}
 }
 
 static gboolean
-update_integer_settings_at_idle (EelPreferencesItem *preferences_item)
+update_integer_settings_at_idle (EelPreferencesItem *item)
 {
 	int value = 0;
 	char *text;
 
-	text = eel_text_caption_get_text (EEL_TEXT_CAPTION (preferences_item->details->main_child));
-
-	if (text != NULL) {
-		eel_eat_str_to_int (text, &value);
-	}
+	text = eel_text_caption_get_text (EEL_TEXT_CAPTION (item->details->main_child));
+	eel_str_to_int (text, &value);
+	g_free (text);
 	
-	eel_preferences_set_integer (preferences_item->details->preference_name, value);
+	eel_preferences_set_integer (item->details->preference_name, value);
 
 	integer_idle_handler = FALSE;
 
@@ -1306,64 +1124,78 @@ update_integer_settings_at_idle (EelPreferencesItem *preferences_item)
 }
 
 static void
-preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem *preferences_item)
+preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem *item)
 {
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item));
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 
 	if (integer_idle_handler == FALSE) {
-		gtk_idle_add ((GtkFunction) update_integer_settings_at_idle, preferences_item);
+		gtk_idle_add ((GtkFunction) update_integer_settings_at_idle, item);
 		integer_idle_handler = TRUE;
 	}
 }
 
+static void
+preferences_item_update_description (EelPreferencesItem *item)
+{
+	char *description;
+
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
+	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
+	
+	description = eel_preferences_get_description (item->details->preference_name);
+	g_return_if_fail (description != NULL);
+	eel_preferences_item_set_description (item, description);
+	g_free (description);
+}
+
 void
-eel_preferences_item_set_control_preference (EelPreferencesItem *preferences_item,
+eel_preferences_item_set_control_preference (EelPreferencesItem *item,
 					     const char *control_preference_name)
 {
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item));
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 
-	if (eel_str_is_equal (preferences_item->details->control_preference_name,
+	if (eel_str_is_equal (item->details->control_preference_name,
 			      control_preference_name)) {
 		return;
 	}
 
-	g_free (preferences_item->details->control_preference_name);
-	preferences_item->details->control_preference_name = g_strdup (control_preference_name);
+	g_free (item->details->control_preference_name);
+	item->details->control_preference_name = g_strdup (control_preference_name);
 }
 
 void
-eel_preferences_item_set_control_action (EelPreferencesItem *preferences_item,
+eel_preferences_item_set_control_action (EelPreferencesItem *item,
 					 EelPreferencesItemControlAction control_action)
 {
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item));
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (control_action >= EEL_PREFERENCE_ITEM_SHOW);
 	g_return_if_fail (control_action <= EEL_PREFERENCE_ITEM_HIDE);
 
-	if (preferences_item->details->control_action == control_action) {
+	if (item->details->control_action == control_action) {
 		return;
 	}
 
-	preferences_item->details->control_action = control_action;
+	item->details->control_action = control_action;
 }
 
 static gboolean
-preferences_item_get_control_showing (const EelPreferencesItem *preferences_item)
+preferences_item_get_control_showing (const EelPreferencesItem *item)
 {
 	gboolean value;
 
-	g_return_val_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item), FALSE);
+	g_return_val_if_fail (EEL_IS_PREFERENCES_ITEM (item), FALSE);
 
-	if (preferences_item->details->control_preference_name == NULL) {
+	if (item->details->control_preference_name == NULL) {
 		return TRUE;
 	}
 
-	value = eel_preferences_get_boolean (preferences_item->details->control_preference_name);
+	value = eel_preferences_get_boolean (item->details->control_preference_name);
 
-	if (preferences_item->details->control_action == EEL_PREFERENCE_ITEM_SHOW) {
+	if (item->details->control_action == EEL_PREFERENCE_ITEM_SHOW) {
 		return value;
 	}
 
-	if (preferences_item->details->control_action == EEL_PREFERENCE_ITEM_HIDE) {
+	if (item->details->control_action == EEL_PREFERENCE_ITEM_HIDE) {
 		return !value;
 	}
 
@@ -1435,17 +1267,17 @@ eel_preferences_item_update_showing (EelPreferencesItem *item)
 void
 eel_preferences_item_enumeration_list_set_unique_exceptions (EelPreferencesItem *item,
 							     const char *exceptions,
-							     const char *exceptions_delimeter)
+							     const char *exceptions_delimiter)
 {
 	EelStringList *new_exceptions;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_LIST_VERTICAL
 			  || item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_LIST_HORIZONTAL);
-	g_return_if_fail (eel_strlen (exceptions_delimeter) > 0);
+	g_return_if_fail (eel_strlen (exceptions_delimiter) > 0);
 
 	new_exceptions = eel_string_list_new_from_tokens (exceptions,
-							  exceptions_delimeter,
+							  exceptions_delimiter,
 							  TRUE);
 
 	if (eel_string_list_equals (new_exceptions, item->details->enumeration_list_unique_exceptions)) {
@@ -1458,3 +1290,91 @@ eel_preferences_item_enumeration_list_set_unique_exceptions (EelPreferencesItem
 
 	preferences_item_update_enumeration_list_uniqueness (item);
 }
+
+void
+eel_preferences_item_set_description (EelPreferencesItem *item,
+				      const char *description)
+{
+	GList *node;
+
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
+	g_return_if_fail (description != NULL);
+
+	switch (item->details->item_type) {
+	case EEL_PREFERENCE_ITEM_BOOLEAN:
+		g_assert (GTK_IS_CHECK_BUTTON (item->details->main_child));
+		g_assert (GTK_IS_LABEL (GTK_BIN (item->details->main_child)->child));
+		gtk_label_set_text (GTK_LABEL (GTK_BIN (item->details->main_child)->child),
+				    description);
+		break;
+	case EEL_PREFERENCE_ITEM_ENUMERATION_VERTICAL_RADIO:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_HORIZONTAL_RADIO:
+		/* These 2 dont have a visible description.  */
+		break;
+	case EEL_PREFERENCE_ITEM_ENUMERATION_LIST_VERTICAL:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_LIST_HORIZONTAL:
+		g_assert (GTK_IS_BOX (item->details->main_child));
+		node = g_list_first (GTK_BOX (item->details->main_child)->children);
+		g_assert (node != NULL);
+		g_assert (node->data != NULL);
+		g_assert (GTK_IS_LABEL (((GtkBoxChild *) node->data)->widget));
+		gtk_label_set_text (GTK_LABEL (((GtkBoxChild *) node->data)->widget),
+				    description);
+		break;
+	case EEL_PREFERENCE_ITEM_FONT:
+	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
+	case EEL_PREFERENCE_ITEM_EDITABLE_INTEGER:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+		g_assert (EEL_IS_CAPTION (item->details->main_child));
+		eel_caption_set_title_label (EEL_CAPTION (item->details->main_child),
+					     description);
+		break;	
+	case EEL_PREFERENCE_ITEM_PADDING:
+		/* Padding doesnt have a description */
+		break;
+	case EEL_PREFERENCE_ITEM_CUSTOM:
+		g_signal_emit (G_OBJECT (item),
+			       signals[CUSTOM_DESCRIPTION_CHANGED], 0,
+			       item->details->main_child,
+			       description);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+}
+
+static void
+eel_preferences_item_class_init (EelPreferencesItemClass *class)
+{
+	G_OBJECT_CLASS (class)->finalize = preferences_item_finalize;
+
+	/* Signals */
+	signals[CUSTOM_UPDATE_DISPLAYED_VALUE] = 
+		g_signal_new ("custom_update_displayed_value",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EelPreferencesItemClass, custom_update_displayed_value),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 
+			      0);
+	signals[CUSTOM_DESCRIPTION_CHANGED] = 
+ 		g_signal_new ("custom_description_changed",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EelPreferencesItemClass, custom_update_description),
+			      NULL, NULL,
+			      eel_marshal_VOID__POINTER_STRING,
+			      G_TYPE_NONE, 
+			      2,
+			      G_TYPE_OBJECT,
+			      G_TYPE_STRING);
+}
+
+static void
+eel_preferences_item_instance_init (EelPreferencesItem *item)
+{
+	item->details = g_new0 (EelPreferencesItemDetails, 1);
+	item->details->item_type = PREFERENCES_ITEM_UNDEFINED_ITEM;
+	item->details->enumeration_list_unique_exceptions = eel_string_list_new (TRUE);
+}
